# -*- coding: utf-8 -*-

"""WebHelpers used in tgapp-categories."""

#from webhelpers import date, feedgenerator, html, number, misc, text
from markupsafe import Markup

def bold(text):
    return Markup('<strong>%s</strong>' % text)


# waiting for a new version of tgext.pluggable
try:
    from tgext.pluggable.utils import instance_primary_key
except ImportError:  # when tgext.pluggable is not updated
    def instance_primary_key(instance, as_string=False):
        """Returns the value of the primary key of the instance"""
        from tgext.pluggable.utils import primary_key
        p = getattr(instance, primary_key(instance.__class__).name)
        return p if not as_string else str(p)



from ming import schema as s
from ming.odm import FieldProperty
from ming.odm.declarative import MappedClass

from tgappcategories.model import DBSession


class Category(MappedClass):
    class __mongometa__:
        session = DBSession
        name = 'tgappcategories_categories'
        indexes = [
            ('name',),
        ]

    _id = FieldProperty(s.ObjectId)
    
    name = FieldProperty(s.String)
    description = FieldProperty(s.String)



# -*- coding: utf-8 -*-
"""The tgapp-categories package"""

from tg.configuration import milestones


def plugme(app_config, options):
    from tgappcategories import model
    milestones.config_ready.register(model.configure_models)
    app_config['_pluggable_tgappcategories_config'] = options
    return dict(appid='tgappcategories', global_helpers=False)


# -*- coding: utf-8 -*-
"""Main Controller"""

from tg import TGController
from tg import expose, flash, require, url, lurl, request, redirect, validate, predicates
from tg.i18n import ugettext as _

from tgappcategories import model
from tgext.pluggable import app_model, plug_url, plug_redirect

from tgappcategories.lib import get_new_category_form, get_edit_category_form


class RootController(TGController):
    allow_only = predicates.has_permission('tgappcategories')

    @expose('tgappcategories.templates.index')
    def index(self):
        categories = model.provider.query(model.Category)
        return dict(categories_count=categories[0],
                    categories=categories[1],
                    mount_point=self.mount_point,
                    )

    @expose('tgappcategories.templates.new_category')
    def new_category(self, **_):
        return dict(form=get_new_category_form(),
                    mount_point=self.mount_point,
                    action=plug_url('tgappcategories', '/create_category'),
                    values=None,
                    )

    @expose()
    @validate(get_new_category_form(), error_handler=new_category)
    def create_category(self, **kwargs):
        dictionary = {
            'name': kwargs.get('name'),
            'description': kwargs.get('description'),
        }
        model.provider.create(model.Category, dictionary)
        flash(_('Category created.'))
        return redirect(url(self.mount_point))

    @expose('tgappcategories.templates.edit_category')
    def edit_category(self, category_id, **_):
        __, categories = model.provider.query(model.Category,
                                              filters={'_id': category_id})
        category = categories[0]
        return dict(form=get_edit_category_form(),
                    mount_point=self.mount_point,
                    action=plug_url('tgappcategories', '/update_category/' + category_id),
                    values=category,
                    )

    @expose()
    @validate(get_edit_category_form(), error_handler=edit_category)
    def update_category(self, category_id, **kwargs):
        __, categories = model.provider.query(model.Category,
                                              filters={'_id': category_id})
        category = categories[0]
        category.name = kwargs.get('name')
        category.description = kwargs.get('description')
        flash(_('Category updated.'))
        return redirect(url(self.mount_point))

    @expose()
    def delete_category(self, category_id):
        model.provider.delete(model.Category, dict(_id=category_id))
        flash(_('Category deleted'))
        return redirect(url(self.mount_point))



from sqlalchemy import Column
from sqlalchemy.types import Unicode, Integer

from sqlalchemy.ext.declarative import declarative_base


DeclarativeBase = declarative_base()


class Category(DeclarativeBase):
    __tablename__ = 'tgappcategories_categories'

    _id = Column(Integer, autoincrement=True, primary_key=True)

    name = Column(Unicode())
    description = Column(Unicode())


from tg.i18n import lazy_ugettext as l_
from tw2.core import Required
from tw2.forms.widgets import Form, TextField, TextArea, SubmitButton
from axf.bootstrap import BootstrapFormLayout


class KajikiBootstrapFormLayout(BootstrapFormLayout):
    inline_engine_name = 'kajiki'


class NewCategory(Form):    
    class child(KajikiBootstrapFormLayout):
        name = TextField(label=l_('Name'), css_class='form-control',
                         validator=Required)

        description = TextArea(label=l_('Description'), rows=10, css_class='form-control',
                               validator=Required)

    submit = SubmitButton(css_class='btn btn-primary pull-right', value=l_('Create'))


class EditCategory(NewCategory):
    submit = SubmitButton(css_class='btn btn-primary pull-right', value=l_('Edit'))



# -*- coding: utf-8 -*-
"""Setup the tgappcategories application"""
from __future__ import print_function

from tgappcategories import model
from tgext.pluggable import app_model


def bootstrap(command, conf, vars):
    print('Bootstrapping tgappcategories...')

    p = app_model.Permission(permission_name='tgappcategories',
                             description='Permits to manage categories')
    try:
        model.DBSession.add(p)
    except AttributeError:
        # mute ming complaints
        pass
    model.DBSession.flush()


import tg
from tgext.pluggable import app_model
from .base import configure_app, create_app, flush_db_changes
from tgappcategories import model
import re
from webtest import AppError
find_urls = re.compile('http[s]?://(?:[a-zA-Z]|[0-9]|[$-_@.&+]|[!*\(\),]|(?:%[0-9a-fA-F][0-9a-fA-F]))+')


class RegistrationControllerTests(object):
    def setup(self):
        self.app = create_app(self.app_config, False)

    def test_index(self):
        resp = self.app.get('/')
        assert 'HELLO' in resp.text

    def test_tgappcategories_index(self):
        resp = self.app.get('/tgappcategories', extra_environ={'REMOTE_USER': 'manager'})

        assert '/new_category' in resp.text, resp

    def test_tgappcategories_auth(self):
        try:
            self.app.get('/tgappcategories')
        except AppError as e:
            assert '401' in str(e)

    def test_create_category(self):
        self.app.get('/tgappcategories/create_category',
                     params={'name': 'category one', 'description': 'pretty description'},
                     extra_environ={'REMOTE_USER': 'manager'},
                     status=302,
                     )
        __, cats = model.provider.query(model.Category,
                                        filters=dict(name='category one'),
                                        )

        assert 'category one' == cats[0].name, cats[0].name
        assert 'pretty description' == cats[0].description, cats[0].description

    def test_update_category(self):
        self.app.get('/tgappcategories/create_category',
                     params={'name': 'category one', 'description': 'pretty description'},
                     extra_environ={'REMOTE_USER': 'manager'},
                     status=302,
                     )
        __, cats = model.provider.query(model.Category,
                                        filters=dict(name='category one'),
                                        )

        self.app.get('/tgappcategories/update_category/' + str(cats[0]._id),
                     params={'name': 'edited category', 'description': 'edited description'},
                     extra_environ={'REMOTE_USER': 'manager'},
                     status=302,
                     )
        __, cats = model.provider.query(model.Category,
                                        filters=dict(name='edited category'),
                                        )

        assert 'edited category' == cats[0].name, cats[0].name
        assert 'edited description' == cats[0].description, cats[0].description

    def test_delete_category(self):
        self.app.get('/tgappcategories/create_category',
                     params={'name': 'category one', 'description': 'pretty description'},
                     extra_environ={'REMOTE_USER': 'manager'},
                     status=302,
                     )
        __, cats = model.provider.query(model.Category,
                                        filters=dict(name='category one'),
                                        )

        self.app.get('/tgappcategories/delete_category/' + str(cats[0]._id),
                     extra_environ={'REMOTE_USER': 'manager'},
                     status=302,
                     )

        count, cats = model.provider.query(model.Category,
                                           filters=dict(name='category one'),
                                           )
        assert count == 0, cats

    def test_new_category_form(self):
        resp = self.app.get('/tgappcategories/new_category', extra_environ={'REMOTE_USER': 'manager'})

        assert 'name="name"' in resp.text, resp
        assert 'name="description"' in resp.text, resp
        assert '/create_category' in resp.text, resp

    def test_edit_category_form(self):
        self.app.get('/tgappcategories/edit_category', extra_environ={'REMOTE_USER': 'manager'}, status=404)

    def test_edit_category_form(self):
        self.app.get('/tgappcategories/create_category',
                     params={'name': 'category one', 'description': 'pretty description'},
                     extra_environ={'REMOTE_USER': 'manager'},
                     status=302,
                     )
        __, cats = model.provider.query(model.Category,
                                        filters=dict(name='category one'),
                                        )

        resp = self.app.get('/tgappcategories/edit_category/' + str(cats[0]._id),
                            extra_environ={'REMOTE_USER': 'manager'}
                            )

        assert 'name="name"' in resp.text, resp
        assert 'name="description"' in resp.text, resp
        assert '/update_category' in resp.text, resp


class TestRegistrationControllerSQLA(RegistrationControllerTests):
    @classmethod
    def setupClass(cls):
        cls.app_config = configure_app('sqlalchemy')


class TestRegistrationControllerMing(RegistrationControllerTests):
    @classmethod
    def setupClass(cls):
        cls.app_config = configure_app('ming')